import amazon
import time
import re

"""
      Asin - Amazon ID ("ASIN" number) of this item
      Authors - list of authors
      Availability - "available", etc.
      BrowseList - list of related categories
      Catalog - catalog type ("Book", etc)
      CollectiblePrice - ?, format "$34.95"
      ImageUrlLarge - URL of large image of this item
      ImageUrlMedium - URL of medium image of this item
      ImageUrlSmall - URL of small image of this item
      Isbn - ISBN number
      ListPrice - list price, format "$34.95"
      Lists - list of ListMania lists that include this item
      Manufacturer - manufacturer
      Media - media ("Paperback", "Audio CD", etc)
      NumMedia - number of different media types in which this item is available
      OurPrice - Amazon price, format "$24.47"
      ProductName - name of this item
      ReleaseDate - release date, format "09 April, 1999"
      Reviews - reviews (AvgCustomerRating, plus list of CustomerReview with Rating, Summary, Content)
      SalesRank - sales rank (integer)
      SimilarProducts - list of Product, which is ASIN number
      ThirdPartyNewPrice - ?, format "$34.95"
      URL - URL of this item
"""

class AmazonData:
    __allow_access_to_unprotected_subobjects__ = 1

    last_updated = 0
    last_update_attempt = 0
    
    def __init__(self, xml=None):
        self.last_updated = 0
        self.last_update_attempt = 0

        self.asin = None
        self.product_name = None
        self.availability = None
        self.catalog_type = None
        self.collectible_price = None
        self.image_url_small = None
        self.image_url_medium = None
        self.image_url_large = None
        self.isbn = None
        self.list_price = None
        self.media = None
        self.manufacturer = None
        self.release_date = None
        self.sales_rank = 999999999
        self.third_party_new_price = None
        self.third_party_used_price = None
        self.amazon_price = None
        self.amazon_url = None

        self.authors = []
        self.browse_list = []
        self.lists = []
        self.reviews = []
        self.avg_customer_rating = None
        self.total_customer_reviews = 0
        self.similar_products = []

        if xml:
            self.init_from_xml(xml)

    def _update_attribute(self, attribute_name, item, item_attribute_name):
        v = getattr(item, item_attribute_name, None)
        if v is not None:
            setattr(self, attribute_name, v)      
    
    def getProductName(self,encoding):
        try:
            return self.product_name.encode(encoding)
        except:
            return self.product_name
        
    def init_from_xml(self, xml):
        """Initialize an AmazonData from the xml returned by a search"""

        self.last_update_attempt = time.time()
        
        if not xml:
            return

        asin = getattr(xml, 'Asin', None)
        if asin is None: # something went wrong
            return
        self.asin = asin

        product_name = getattr(xml, 'ProductName', None)
        if product_name is None: # something went wrong
            return

        self.product_name =product_name

        self._update_attribute('availability', xml, 'Availability')
        self._update_attribute('catalog_type', xml, 'Catalog')
        self._update_attribute('collectible_price', xml, 'CollectiblePrice')
        self._update_attribute('image_url_small', xml, 'ImageUrlSmall')
        self._update_attribute('image_url_medium', xml, 'ImageUrlMedium')
        self._update_attribute('image_url_large', xml, 'ImageUrlLarge')
        self._update_attribute('isbn', xml, 'Isbn')
        self._update_attribute('list_price', xml, 'ListPrice')
        self._update_attribute('media', xml, 'Media')
        self._update_attribute('manufacturer', xml, 'Manufacturer')
        self._update_attribute('release_date', xml, 'ReleaseDate')
        self._update_attribute('sales_rank', xml, 'SalesRank')
        self._update_attribute('third_party_new_price', xml, 'ThirdPartyNewPrice')
        self._update_attribute('third_party_used_price', xml, 'UsedPrice')
        self._update_attribute('amazon_price', xml, 'OurPrice')
        self._update_attribute('amazon_url', xml, 'URL')
        
        authors = getattr(xml, 'Authors', None)
        if authors != None:
            authors = getattr(authors, 'Author', [])
            if type(authors) != type([]):
                authors = [authors]
            self.authors = authors

        browse_list = getattr(xml, 'BrowseList', None)
        if browse_list != None:
            browse_list = getattr(browse_list, 'BrowseNode', [])
            if type(browse_list) != type([]):
                browse_list = [browse_list]
            browse_list = [b.BrowseName for b in browse_list if getattr(b, 'BrowseName', '')]
            self.browse_list = browse_list

        lists = getattr(xml, 'Lists', None)
        if lists != None:
            lists = getattr(lists, 'ListId', [])
            if type(lists) != type([]):
                lists = [lists]
            self.lists = lists

        reviews = getattr(xml, 'Reviews', None)
        if reviews != None:
            avg_customer_rating = getattr(reviews, 'AvgCustomerRating', None)
            try:
                avg_customer_rating = float(avg_customer_rating)
            except (ValueError, TypeError):
                avg_customer_rating = None
            total_customer_reviews = getattr(reviews, 'TotalCustomerReviews', 0)
            try:
                total_customer_reviews = int(total_customer_reviews)
            except (ValueError, TypeError):
                total_customer_reviews = 0
            customer_reviews = getattr(reviews, 'CustomerReview', [])
            review_list = []
            if type(customer_reviews) != type([]):
                customer_reviews = [customer_reviews]
            for cr in customer_reviews:
                review_list.append(AmazonReview(cr))
            self.reviews = review_list
            self.avg_customer_rating = avg_customer_rating
            self.total_customer_reviews = total_customer_reviews

        sp = getattr(xml, 'SimilarProducts', None)
        if sp != None:
            sp = getattr(sp, 'Product', [])
            if type(sp) != type([]):
                sp = [sp]
            self.similar_products = sp

        self.last_updated = time.time()
    
    
class AmazonReview:
    __allow_access_to_unprotected_subobjects__ = 1
    
    def __init__(self, customer_review):
        self.summary = getattr(customer_review, 'Summary', '')
        self.comment = getattr(customer_review, 'Comment', '')
        rating = getattr(customer_review, 'Rating', None)
        try:
            rating = int(rating)
        except (ValueError, TypeError):
            rating = None
        self.rating = rating

    def getSummary(self):
        return self.summary
    
    def getComment(self):
        return self.comment
    
    def getRating(self):
        # rating should be an integer or None
        return self.rating
