import amazon
import os
import time

from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner
import Globals
from ZODB.POSException import ConflictError

from Products.CMFCore.utils import getToolByName, UniqueObject, SimpleItemWithProperties
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from SimpleCache import SimpleCache
from config import HEAVY_CACHE_MAX_AGE, HEAVY_CACHE_MAX_SIZE, \
                   LITE_CACHE_MAX_AGE, LITE_CACHE_MAX_SIZE, \
                   POWER_CACHE_MAX_AGE, POWER_CACHE_MAX_SIZE, \
                   CACHE_CLEANUP_INTERVAL
from AmazonItem import AmazonItem
from AmazonData import AmazonData

class AmazonTool(UniqueObject, SimpleItemWithProperties):

    security = ClassSecurityInfo()

    id = 'amazon_tool'
    title = 'Manage Amazon items on the site'
    meta_type= 'Amazon Tool'

    manage_options = ( ({'label':'Overview', 'action':'manage_overview'},) +
                       SimpleItemWithProperties.manage_options )

    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile(os.path.join('www','manage_overview'), globals())
    manage_overview.__name__ = 'manage_overview'
    manage_overview._need__name__ = 0

    _properties = ({'id':'amazon_developer_key', 'type':'string', 'mode':'w',
                    'label':'Amazon web services developer key (AWS 3.0)'},
                   {'id':'amazon_associate_id', 'type':'string', 'mode':'w',
                    'label':'Amazon associate ID'},
                   {'id':'amazon_locale', 'type':'selection', 'mode':'w',
                    'select_variable':'amazon_supported_locales',
                    'label':'Amazon Locale'},
                   {'id':'amazon_item_folder_id', 'type':'string', 'mode':'w',
                    'label':'Amazon item folder ID'},
                   {'id':'n_items_in_portlet', 'type':'int', 'mode':'w',
                    'label':'Number of items listed in the Amazon portlet'},
                   {'id':'show_similar_items', 'type':'selection', 'mode':'w',
                    'select_variable':'yes_no',
                    'label':'Show similar items in the portlet when viewing an item'},
                   )

    amazon_developer_key = ''
    amazon_associate_id = amazon.getAssociate()
    amazon_locale = amazon.getLocale()
    amazon_supported_locales = amazon._supportedLocales.keys()
    amazon_item_folder_id = 'books'
    n_items_in_portlet = 5
    show_similar_items = 'yes'
    yes_no = ['yes', 'no']

    # note that all instances of AmazonTool will share a common cache
    
    _heavy_cache = SimpleCache(HEAVY_CACHE_MAX_AGE, HEAVY_CACHE_MAX_SIZE, CACHE_CLEANUP_INTERVAL)  # cache for objects obtained via a heavy search
    _lite_cache = SimpleCache(LITE_CACHE_MAX_AGE, LITE_CACHE_MAX_SIZE, CACHE_CLEANUP_INTERVAL)   # cache for objects obtained via a "lite" search
    _power_cache = SimpleCache(POWER_CACHE_MAX_AGE, POWER_CACHE_MAX_SIZE, CACHE_CLEANUP_INTERVAL)   # cache for objects obtained via a power search

    security.declarePrivate('getDeveloperKey')
    def getDeveloperKey(self):
        amazon_developer_key = self.getProperty('amazon_developer_key', None)
        if not amazon_developer_key:
            raise ValueError, 'Set your Amazon web services developer key in amazon_tool'
        return amazon_developer_key.strip()

    security.declarePublic('getAssociateId')
    def getAssociateId(self):
        amazon_associate_id = self.getProperty('amazon_associate_id', None)
        if not amazon_associate_id:
            raise ValueError, 'Set your Amazon associate ID in amazon_tool'
        return amazon_associate_id.strip()

    security.declarePublic('getLocale')
    def getLocale(self):
        amazon_locale = self.getProperty('amazon_locale', None)
        if not amazon_locale:
            raise ValueError, 'Set your Amazon locale in amazon_tool'
        return amazon_locale.strip()
    
    def getCache(self, heavy):
        if heavy:
            return self._heavy_cache
        return self._lite_cache

    def search_by_asin(self, asin, heavy):
        search_type = heavy and 'heavy' or 'lite'
        try:
            matches = amazon.searchByASIN(asin, type=search_type,
                                          license_key=self.getDeveloperKey(),
                                          locale = self.getLocale(),
                                          associate = self.getAssociateId())
            return [AmazonData(m) for m in matches]
        except ConflictError:
            raise
        except (amazon.AmazonError, IOError):
            pass
        except: # catch xml parse errors (ExpatError), etc
            pass
        return []

    def cached_search_by_asin(self, asin, heavy=True):
        matches = self.getCache(heavy).get(asin)
        if not matches and not heavy:
            # we can use xml from the heavy cache for lite requests
            matches = self.getCache(heavy=True).get(asin)
            
        if not matches:
            matches = self.search_by_asin(asin, heavy)
            if matches:
                self.getCache(heavy).set(asin, matches)
        return matches

    def _get_power_search_key(self, author, title, keywords, subject, isbn, publisher):
        s = ''
        if author:
            s += 'author: ' + author.strip()
        if title:
            s += 'title: ' + title.strip()
        if keywords:
            s += 'keywords: ' + keywords.strip()
        if subject:
            s += 'subject: ' + subject.strip()
        if isbn:
            s += 'isbn: ' + isbn.strip()
        if publisher:
            s += 'publisher: ' + publisher.strip()
        return s
        
    def power_search(self, author='', title='', keywords='', subject='', isbn='', publisher='', search_type='heavy'):
        s = self._get_power_search_key(author, title, keywords, subject, isbn, publisher)
        try:
            matches = amazon.searchByPower(s, type=search_type,
                                           license_key=self.getDeveloperKey(),
                                           locale = self.getLocale(),
                                           associate = self.getAssociateId())
        except amazon.AmazonError, e:
            if str(e).startswith('There are no exact matches for the search'):
                matches = []
            else:
                raise
        return [AmazonData(m) for m in matches]

    def cached_power_search(self, author='', title='', keywords='', subject='', isbn='', publisher='', search_type='heavy'):
        cache = self._power_cache
        s = self._get_power_search_key(author, title, keywords, subject, isbn, publisher) + ' search_type='+search_type
            
        matches = cache.get(s)
        if not matches:
            matches = self.power_search(author, title, keywords, subject, isbn, publisher, search_type)
            if matches:
                cache.set(s, matches)
        return matches

    security.declarePublic('searchByAsin')
    def searchByAsin(self, asin, search_type='heavy', REQUEST=None):
        if REQUEST is not None:
            return # prevent calling search via a URL
        matches = self.cached_search_by_asin(asin, search_type)
        if not matches:
            return None
        ad = matches[0]
        item = AmazonItem(asin).__of__(self)
        item.initializeArchetype()
        item._update(search_type=='heavy') # force update of object
        return 

    security.declarePublic('powerSearch')
    def powerSearch(self, author='', title='', keywords='', subject='', isbn='', publisher='', search_type='heavy', REQUEST=None):
        if REQUEST is not None:
            return # prevent calling search via a URL
        matches = self.cached_power_search(author, title, keywords, subject, isbn, publisher, search_type)
        
        items = []
        n = 0
        for ad in matches:
            item = AmazonItem(str(ad.asin)).__of__(self)
            item.initializeArchetype()
            item.asin = ad.asin
            item._update(search_type=='heavy')  # force update of object
            items.append(item)
            n += 1
        return items

    def __getitem__(self, id):
        try:
            return SimpleItemWithProperties.__getitem(self, id)
        except AttributeError:
            pass
        ad = self.cached_search_by_asin(id, True)
        if not ad:
            raise AttributeError, id
        item = AmazonItem(id).__of__(self)
        item.initializeArchetype()
        item.asin = id
        item._update(True)  # force update of object
        intended_parent = aq_parent(self)
        at = aq_base(self).__of__(intended_parent)  # rewrap self
        return item.__of__(at)

Globals.InitializeClass(AmazonTool)
